# include "../main/utilities.h"

# include "formula.h"


Formula::Formula() {
	formulasIncrement(); }


Formula::~Formula() {
	formulasDecrement(); }


/*
 * Returns the number of elements of this formula.
 *
 * This is the default fuction. Always returns 0.
 */
int Formula::getNumberOfElements() {
	return 0; }


/*
 * @param index: the index of the element to be retrieved.
 * Returns the element at @index of this formula, if possible.
 * Returns NULL otherwise.
 *
 * This is the default fuction. Always returns NULL.
 */
Formula* Formula::getElement (int index) {
	return NULL; }


/*
 * Returns the IDValue of this formula, if possible.
 * Returns -1 otherwise.
 *
 * This is the default fuction. Always returns -1.
 */
int Formula::getIDValue () {
	return -1; }


/*
 * @param f: the formula to be compared.
 * Returns true if this instance equals f. Returns false otherwise.
 *
 * This is the default fuction. Always returns false.
 */
bool Formula::equals (Formula *f) {
	return false; }


/*
 * @param v: the assignment to be applied.
 * Returns the value of this formula, when v is applied.
 * Returns false if no formula is pointed.
 *
 * This is the default fuction. Always returns false.
 */
bool Formula::getValue (Assignment *v) {
	return false; }


/*
 * prints this formula in the standard output.
 *
 * This is the default fuction. It does nothing.
 */
void Formula::print() { }


/*
 * Clear and delete recursively all subformulas. This instance isn't
 * deleted.
 *
 * This is the default fuction. It does nothing.
 */
void Formula::clear() { }


/*
 * erases recursively this formula and all subformulas.
 */
void Formula::erase() {
	clear();
	delete this; }


FormulaTrue::FormulaTrue () : Formula () { }


FormulaTrue::~FormulaTrue () { }


/*
 * @param f: the formula to be compared.
 * Returns true if this instance equals f. Returns false otherwise.
 */
bool FormulaTrue::equals (Formula *f) {
	return dynamic_cast<FormulaTrue *>(f) != NULL; }


/*
 * @param v: the assignment to be applied.
 * Returns the value of this formula, when v is applied.
 * Returns false if no formula is pointed.
 */
bool FormulaTrue::getValue (Assignment *v) {
	return true; }


/*
 * prints this formula in the standard output.
 */
void FormulaTrue::print() {
	std::cout << "1"; }


FormulaFalse::FormulaFalse () : Formula () { }


FormulaFalse::~FormulaFalse () { }


/*
 * @param f: the formula to be compared.
 * Returns true if this instance equals f. Returns false otherwise.
 */
bool FormulaFalse::equals (Formula *f) {
	return dynamic_cast<FormulaFalse *>(f) != NULL; }


/*
 * @param v: the assignment to be applied.
 * Returns the value of this formula, when v is applied.
 * Returns false if no formula is pointed.
 */
bool FormulaFalse::getValue (Assignment *v) {
	return false; }


/*
 * prints this formula in the standard output.
 */
void FormulaFalse::print() {
	std::cout << "0"; }


FormulaIdentif::FormulaIdentif (int IDValue) : Formula () {
	this->IDValue = IDValue; }


FormulaIdentif::~FormulaIdentif () { }


/*
 * Returns the IDValue of this formula, if possible.
 * Returns -1 otherwise.
 */
int FormulaIdentif::getIDValue () {
	return IDValue; }


/*
 * @param f: the formula to be compared.
 * Returns true if this instance equals f. Returns false otherwise.
 */
bool FormulaIdentif::equals (Formula *f) {
	FormulaIdentif *fi = dynamic_cast<FormulaIdentif *>(f);

	if (fi == NULL) return false;
	
	return this->IDValue == fi->IDValue; }


/*
 * @param v: the assignment to be applied.
 * Returns the value of this formula, when v is applied.
 * Returns false if no formula is pointed.
 */
bool FormulaIdentif::getValue (Assignment *v) {
	return v->getValue(IDValue); }


/*
 * prints this formula in the standard output.
 */
void FormulaIdentif::print() {
	std::cout << "va_" << IDValue; }


FormulaBox::FormulaBox (Formula *element) : Formula () {
	this->element = element; }


FormulaBox::~FormulaBox () { }


/*
 * Returns the number of elements of this formula.
 */
int FormulaBox::getNumberOfElements() {
	return 1; }


/*
 * @param index: the index of the element to be retrieved.
 * Returns the element at @index of this formula, if possible.
 * Returns NULL otherwise.
 */
Formula* FormulaBox::getElement (int index) {
	if (index == 0) return element;
	return NULL; }


/*
 * @param f: the formula to be compared.
 * Returns true if this instance equals f. Returns false otherwise.
 */
bool FormulaBox::equals (Formula *f) {
	FormulaBox *fb = dynamic_cast<FormulaBox *>(f);

	if (fb == NULL) return false;
	if (element == NULL) {
		if (fb->element != NULL) return false; }
	else if (!(element->equals(fb->element))) return false;

	return true; }


/*
 * @param v: the assignment to be applied.
 * Returns the value of this formula, when v is applied.
 * Returns false if no formula is pointed.
 */
bool FormulaBox::getValue (Assignment *v) {
	if (element != NULL) return element->getValue(v);
	return false; }


/*
 * prints this formula in the standard output.
 */
void FormulaBox::print() {
	if (element == NULL) return;

	std::cout << "[](";
	element->print();
	std::cout << ")"; }


/*
 * Clear and delete recursively all subformulas. This instance isn't
 * deleted.
 */
void FormulaBox::clear() {
	if (element != NULL) element->clear();
	delete element; }


FormulaAnd::FormulaAnd (std::vector<Formula*> *elements) : Formula () {
	std::vector<Formula*>::iterator it;

	this->elements = new std:: vector<Formula*>();
	if (elements != NULL) {
		for (it = elements->begin(); it != elements->end(); it++) {
			this->elements->push_back(*(it)); }}}


FormulaAnd::~FormulaAnd () {
	delete elements; }


/*
 * Returns the number of elements of this formula.
 */
int FormulaAnd::getNumberOfElements() {
	return elements->size(); }


/*
 * @param index: the index of the element to be retrieved.
 * Returns the element at @index of this formula, if possible.
 * Returns NULL otherwise.
 */
Formula* FormulaAnd::getElement (int index) {
	if (index < 0 || index >= (signed) elements->size()) return NULL;
	return (*elements)[index]; }


/*
 * @param f: the formula to be compared.
 * Returns true if this instance equals f. Returns false otherwise.
 */
bool FormulaAnd::equals (Formula *f) {
	FormulaAnd *fa = dynamic_cast<FormulaAnd *>(f);

	if (fa == NULL) return false;
	if (fa->elements->size() != elements->size()) return false;

	for (int i = 0; i < (signed) elements->size(); i++) {
		if (!((*elements)[i]->equals((*(fa->elements))[i]))) return false; }

	return true; }


/*
 * @param v: the assignment to be applied.
 * Returns the value of this formula, when v is applied.
 * Returns false if no formula is pointed.
 */
bool FormulaAnd::getValue (Assignment *v) {
	for (int i = 0; i < (signed) elements->size(); i++) {
		if ((*elements)[i]->getValue(v) == false) return false; }

	return true; }


/*
 * prints this formula in the standard output.
 */
void FormulaAnd::print() {
	std::vector<Formula*>::iterator it = elements->begin();
	if (it == elements->end()) return;

	std::cout << "(";
	while (it != elements->end()) {
		(*it)->print();
		it++;
		if (it != elements->end()) std::cout << "."; }

	std::cout << ")"; }


/*
 * Clear and delete recursively all subformulas. This instance isn't
 * deleted.
 */
void FormulaAnd::clear() {
	std::vector<Formula*>::iterator it;

	for (it = elements->begin(); it != elements->end(); it++) {
		(*(it))->clear();
		delete (*(it)); }}


FormulaXor::FormulaXor (std::vector<Formula*> *elements) : Formula () {
	std::vector<Formula*>::iterator it;

	this->elements = new std:: vector<Formula*>();
	if (elements != NULL) {
		for (it = elements->begin(); it != elements->end(); it++) {
			this->elements->push_back(*(it)); }}}


FormulaXor::~FormulaXor () {
	delete elements; }


/*
 * Returns the number of elements of this formula.
 */
int FormulaXor::getNumberOfElements() {
	return elements->size(); }


/*
 * @param index: the index of the element to be retrieved.
 * Returns the element at @index of this formula, if possible.
 * Returns NULL otherwise.
 */
Formula* FormulaXor::getElement (int index) {
	if (index < 0 || index >= (signed) elements->size()) return NULL;
	return (*elements)[index]; }


/*
 * @param f: the formula to be compared.
 * Returns true if this instance equals f. Returns false otherwise.
 */
bool FormulaXor::equals (Formula *f) {
	FormulaXor *fx = dynamic_cast<FormulaXor *>(f);

	if (fx == NULL) return false;
	if (fx->elements->size() != elements->size()) return false;

	for (int i = 0; i < (signed) elements->size(); i++) {
		if (!((*elements)[i]->equals((*(fx->elements))[i]))) return false; }

	return true; }


/*
 * @param v: the assignment to be applied.
 * Returns the value of this formula, when v is applied.
 * Returns false if no formula is pointed.
 */
bool FormulaXor::getValue (Assignment *v) {
	bool result = false;

	int i;
	for (i = 0; i < (signed) elements->size(); i++) {
		if (result == (*elements)[i]->getValue(v)) result = false;
		else result = true; }

	return result; }


/*
 * prints this formula in the standard output.
 */
void FormulaXor::print() {
	std::vector<Formula*>::iterator it = elements->begin();
	if (it == elements->end()) return;

	std::cout << "(";
	while (it != elements->end()) {
		(*it)->print();
		it++;
		if (it != elements->end()) std::cout << " + "; }

	std::cout << ")"; }


/*
 * Clear and delete recursively all subformulas. This instance isn't
 * deleted.
 */
void FormulaXor::clear() {
	std::vector<Formula*>::iterator it;

	for (it = elements->begin(); it != elements->end(); it++) {
		(*(it))->clear();
		delete (*(it)); }}


Sequent::Sequent (std::vector<Formula*> *premises, Formula *conclusion) :
		Formula () {
	std::vector<Formula*>::iterator it;

	this->conclusion = conclusion;
	this->premises = new std:: vector<Formula*>();
	if (premises != NULL) {
		for (it = premises->begin(); it != premises->end(); it++) {
			this->premises->push_back(*(it)); }}}


Sequent::~Sequent () {
	delete premises; }


/*
 * @param f: the formula to be compared.
 * Returns true if this instance equals f. Returns false otherwise.
 */
bool Sequent::equals (Formula *f) {
	Sequent *s = dynamic_cast<Sequent *>(f);

	if (s == NULL) return false;
	if (conclusion == NULL) {
		if (s->conclusion != NULL) return false; }
	else if (!conclusion->equals(s->conclusion)) return false;

	if (s->premises->size() != premises->size()) return false;

	for (int i = 0; i < (signed) premises->size(); i++) {
		if (!((*premises)[i]->equals((*(s->premises))[i]))) return false; }

	return true; }


/*
 * @param v: the assignment to be applied.
 * Returns the value of this formula, when v is applied.
 * Returns false if no formula is pointed.
 */
bool Sequent::getValue (Assignment *v) {
	if (conclusion->getValue(v) == true) return true;
	std::vector<Formula*>::iterator it;
	for (it = premises->begin(); it != premises->end(); it++) {
		if ((*(it))->getValue(v) == false) return true; }

	return false; }


/*
 * prints this formula in the standard output.
 */
void Sequent::print() {
	std::vector<Formula*>::iterator it = premises->begin();

	while (it != premises->end()) {
		(*it)->print();
		it++;
		if (it != premises->end()) std::cout << "."; }

	if (conclusion != NULL) {
		std::cout << ".(";
		conclusion->print();
		std::cout << " + 1)"; }}


/*
 * Clear and delete recursively all subformulas. This instance isn't
 * deleted.
 */
void Sequent::clear() {
	std::vector<Formula*>::iterator it;

	conclusion->clear();
	delete conclusion;
	for (it = premises->begin(); it != premises->end(); it++) {
		(*(it))->clear();
		delete (*(it)); }}
