# ifndef PARSER_H
# define PARSER_H

# include "../formula/formulaTable.h"
# include "../map/IDmap.h"
# include "stack.h"



typedef enum token {
	TOKEN_OPEN,
	TOKEN_CLOSE,
	TOKEN_AND,
	TOKEN_OR,
	TOKEN_BOX,
	TOKEN_NOT,
	TOKEN_IMPLY,
	TOKEN_IDENTIF,
	TOKEN_COMMA,
	TOKEN_DASH,
	TOKEN_TERMINAL,  // indicates the end of the formula
	TOKEN_ERROR } TokenType;


/**
 *
 * This class parser a string to a formula
 */
class Parser {
	public:
		/*
		 * @param s: the string to be parsed in a formula.
		 * Creates a parser that can parse the requested string.
		 *
		 * Note that the parsing isn't done imediatly.
		 */
		Parser(std::string &s);
		virtual ~Parser();

		/*
		 * Parses the string and returns the identified formula.
		 * Returns NULL if no/incorrect formula is identified in the string.
		 */
		Formula* parse();
		/*
		 * Returns NULL if no/incorrect formula is identified in the string.
		 */
		Map* getMap();

	private:
		bool getEntireFormula();
		bool getFormula();
		bool getSequent(TokenType firstToken);
		bool getComposedFormula();
		bool getImplyFormula ();
		bool getAndFormula ();
		bool getOrFormula ();
		bool getBoxFormula ();
		bool getNotFormula ();
		bool getIdentifierFormula();

		bool isBlank(char c);
		void readBlank();
		char nextChar();
		bool nextCharIsDigit();
		bool nextCharIsLetter();
		std::string* clearBadChars(std::string &s);
		TokenType getNextToken();
		void rollBackLastToken();

		bool nextTokenIsComma();
		bool nextTokenIsDash();
		bool nextTokenIsOpen();
		bool nextTokenIsClose();
		bool nextTokenIsImply();
		bool nextTokenIsAnd();
		bool nextTokenIsOr();
		bool nextTokenIsBox();
		bool nextTokenIsNot();
		bool nextTokenIsTerminal();

		std::string tokenString(TokenType t);

		virtual Formula* createFormulaTrue();
		virtual Formula* createFormulaFalse();
		virtual Formula* createFormulaIdentif(int IDValue);
		virtual Formula* createFormulaBox(Formula *element);
		virtual Formula* createFormulaAnd(std::vector<Formula*> *elements);
		virtual Formula* createFormulaXor(std::vector<Formula*> *elements);
		virtual Formula* createSequent(std::vector<Formula*> *premises,
				Formula *conclusion);

		std::string s;
		int read, lastTokenInit;
		Stack *formulaStack;
		Map *mapper; };



/**
 *
 * This class parser a string to a formula, using the global formula table
 */
class ParserTable: public Parser{
	public:
		ParserTable(std::string &s);
		~ParserTable();

	private:
		Formula* createFormulaTrue();
		Formula* createFormulaFalse();
		Formula* createFormulaIdentif(int IDValue);
		Formula* createFormulaBox(Formula *element);
		Formula* createFormulaAnd(std::vector<Formula*> *elements);
		Formula* createFormulaXor(std::vector<Formula*> *elements);
		Formula* createSequent(std::vector<Formula*> *premises,
				Formula *conclusion); };



# endif //PARSER_H
